#include <avr/io.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>


#include "./driver/nokia3310library/lcd.h"
#include "./driver/keys.h"
#include "./driver/backlight.h"
#include "./driver/uart.h"
#include "./driver/timer1_timebase.h"
#include "./menu/menu.h"
#include "./menu/displayroutines.h"
#include "./apps/apps.h"
#include "./apps/realtimeclock.h"
#include "./apps/clocktimer.h"
#include "./types/mytypes.h"

#ifndef F_CPU
/* Definiere F_CPU, wenn F_CPU nicht bereits vorher definiert
   (z.B. durch bergabe als Parameter zum Compiler innerhalb
   des Makefiles). Zustzlich Ausgabe einer Warnung, die auf die
   "nachtrgliche" Definition hinweist */
#warning "F_CPU war noch nicht definiert, wird nun mit 3686400 definiert"
#define F_CPU 4000000UL     /* Quarz mit 3.6864 Mhz */
#endif




uint8_t onesecondover;
uint8_t oneminuteover;
uint8_t idle;

//void Beep(){
//    PORTD |= (1 << Buzzer);
//    _delay_ms(1);
//    PORTD &=~(1 << Buzzer);
//}

void SetPorts(){
	//Ausgnge am PORTD
	//PORTD = alle Bits nach Reset auf 0
	DDRD = 0xff;
}

t_Menu SettingsMenu={
    .Text = "Settings",
    .menu_id = 1,
    .Previous = NULL,
    .Count = 2,
    .MenuItems = {
        {
        .Text = "BackLight",
        .item_id = 15,
        },
        {
        .Text = "Set Time",
        .item_id = 14,
        },
    },
};

t_Menu EditSwitchpoints ={
    .Text = "Edit Switp",
    .menu_id = 2,
    .Previous = NULL,
    .Count = SWITCHPOINTS_COUNT,
    .ItemType = 1
};

t_Menu MainMenu ={
    .Text = "Hauptmenue",
    .menu_id = 0,
    .Previous = NULL,
    .Count = 3,
    .MenuItems = {
        {
        .Text = "@Settings",
        .item_id = 16,
        },
        {
        .Text = "@Edit Switp",
        .item_id = 19,
        },
        {
        .Text = "Fertig",
        .item_id = 20,
        },
    },
};

t_Menu *Menus[] = {
    &MainMenu,
    &SettingsMenu,
    &EditSwitchpoints,
};


uint8_t MenuCount = 3;

t_ActiveApplication ActiveApplication = &MenuApplication;


void MySelect(uint8_t MenuId, uint8_t action, uint8_t ItemType){
    switch(ItemType){
        case MENU_ITEMTYPE_BUILDIN:{
            switch( MenuId){
                case 14:{
                    Clear();
                    SetDateTimeToChange(RealTime_GetDateTime());
                    ActiveApplication = &SetTimeApp;
                    ActiveApplication(NO_KEY);
                }
                break;
                case 15:{
                    //Beep();
                    Clear();
                    PrintStringPos(0,0,"BackLight");
                    Highlight(1);
                    PrintStringPos(1,1,"Value:");
                    //setBackLightPWMValue(&BackLightPWM);
                    ActiveApplication = &BackLightApplication;
                    ActiveApplication(NO_KEY);
                }
                break;
                case 20:{
                    idle = 1;
                }
                break;
                default:
                break;
            }
        }
        break;
        case 1:{//my special menu type for eeprom array
            if(action == MENU_ACTION_SELECT_SPECIALITEM){
                switch(MenuId){
                    case 2:{
                        Clear();
                        ActiveApplication = &EditSwitchpointApp;
                        ActiveApplication(NO_KEY);
                    }
                    break;
                    default:
                    break;
                }
            }
        }
        break;
    }
}



void TimebaseTick(void){
    DebounceSwitch();
}

void OneSecondTick(void){
    RealTime_OneSecondOver();
    onesecondover = 1;
}

void OneMinuteTick(void){
    oneminuteover = 1;
    idle = 1;
}

int main(void) __attribute__((noreturn));
int main(void){
	SetPorts();
	InitTimer1_Timebase();
    InitKeys();
	lcd_init();
	lcd_contrast(0x40);
    InitBackLight();
	//InitUart();

	EnterMenu(&MainMenu, &MySelect);
	sei();
	for(;;){
       if (onesecondover){
            onesecondover = 0;
            ActiveApplication(TICK);
        }
        if (get_key_short((1<<DOWN))){
            idle = 0;
            ActiveApplication(DOWN);
        }
        if (get_key_short((1<<LEFT))){
            idle = 0;
            ActiveApplication(LEFT);
        }
        if (get_key_short((1<<RIGHT))){
            idle = 0;
            ActiveApplication(RIGHT);
        }
        if (get_key_short((1<<OK))){
            idle = 0;
            ActiveApplication(OK);
        }
        if (get_key_short((1<<UP))){
            idle = 0;
            ActiveApplication(UP);
        }
        if(oneminuteover){
            oneminuteover = 0;
            if(ActiveApplication != EditSwitchpointApp){ //nicht aufrufen wenn Schaltpunkte editiert werden
                IdleApplication(ONE_MINUTE_TICK);
            }
        }
        if(idle){
            idle = 0;
//            if(ActiveApplication == MenuApplication){//nur im Men Idle
//                //display Schaltzustand
                IdleApplication(NO_KEY);
//            }
        }
 	}
////	 Print on first line
//
//	lcd_goto_xy(1,1);
//	lcd_str("RW electronics");
//
//	// Print on second line with custom characters
//	lcd_goto_xy(4,2);
//	lcd_chr(ARROW_LEFT);
//	lcd_str(" 2008 ");
//	lcd_chr(ARROW_RIGHT);
//	lcd_goto_xy(1,4);
// 	//lcd_str("12345678901234");
//	lcd_chr_invers(' ');
//	lcd_chr_invers('d');
//	lcd_chr_invers('a');
//	lcd_chr_invers('s');
//	lcd_chr_invers(' ');
//	lcd_chr_invers('i');
//	lcd_chr_invers('s');
//	lcd_chr_invers('t');
//	lcd_chr_invers(' ');
//	lcd_chr_invers('c');
//	lcd_chr_invers('o');
//	lcd_chr_invers('o');
//	lcd_chr_invers('l');
//	lcd_chr_invers('!');
//	lcd_str(" Code::Blocks");
//
//
//	while (true) {
//		PORTD ^= (1 << BackLight);
//		//PORTD ^= (1 << Buzzer);
//		long_delay(1);
}
